/*
 * BP_ISO15693.c
 *
 *  Created on: 10-03-2013
 *      Author: Maciej
 *
 *      Based on ISO/IEC FCD 15693-3
 *
 *
 *
 */

#include "utils/uartstdio.h"
#include "BP_RFID_TRF.h"
#include "BP_ISO15693.h"
#include "TRF797x.h"

void BP_ISO15693_IRQ(char reg)
{
	if (reg & TRF_IRQ_NORESP) UARTprintf("No response interrupt");
	if (reg & TRF_IRQ_COL)
	{
		//BP_RFID_RX_BUFFER_COUNT =0;
		BP_RFID_TRF_FIFO_Reset();
	}

	if (reg & TRF_IRQ_ERR3)
	{
		UARTprintf("RX framing or EOF error");
		BP_RFID_TRF_FIFO_Reset();
	}
	if (reg & TRF_IRQ_ERR2)
	{
		UARTprintf("RX parity error (ISO14443A)");
		BP_RFID_TRF_FIFO_Reset();
	}
	if (reg & TRF_IRQ_ERR1)
	{
		UARTprintf("RX CRC Error");
		BP_RFID_TRF_FIFO_Reset();
	}

}

// --- Mandatory commands --------------------------------------------------------------------------------------

/// Performs anticollision sequence with single VICC in the field.
/// Returns TRF_NO_ERROR if single VICC successfully responds.
/// UID consists 8 bytes: 'E0', Manufacturer code and serial number
///
/// 9.2.1 Inventory
///
char BP_ISO15693_Inventory_Single_Slot(char* UID)
{
	int i;
	char buf[3];

	buf[0] = ISO15693_FL0_Data_rate_flag | ISO15693_FL0_Inventory_flag | ISO15693_FL2_Single_Slot; //
	buf[1] = ISO15693_CMD_Inventory;
	buf[2] = 0;
	BP_RFID_TRF_Transmit(buf, 3);

	// If timeout, return error
	if (BP_RFID_TRF_Wait_For_Rx_End() == TRF_ERROR) return TRF_ERROR;

	// If some error
	if ((BP_RFID_RX_BUFFER_COUNT != 10) || (BP_RFID_RX_BUFFER[0] != 0)) return TRF_ERROR;

	for (i = 0; i < 8; ++i)
		UID[i] = BP_RFID_RX_BUFFER[9 - i];
	BP_RFID_RX_BUFFER_COUNT = 0;

	return TRF_NO_ERROR;
}

// Reads 4 bytes
//
//
char BP_ISO15693_Read_Single_Block(char* data, char block)
{
	int i;
	char buf[3];

	buf[0] = ISO15693_FL0_Data_rate_flag; //ISO15693_FL0_Data_rate_flag
	buf[1] = ISO15693_CMD_Read_Single_Block;
	buf[2] = block;
	BP_RFID_TRF_Transmit(buf, 3);

	data[0] = 0;

	if ((BP_RFID_TRF_Wait_For_Rx_End() == TRF_ERROR) || (BP_RFID_RX_BUFFER[0] != 0))
	{
		return TRF_ERROR;
	}

	if (BP_RFID_RX_BUFFER_COUNT < 4)
	{
		return TRF_ERROR;
	}

	for (i = 1; i < 5; ++i)
	{
		// debug print of buffer
		//UARTprintf("%02x", BP_RFID_RX_BUFFER[i]);
		data[i - 1] = BP_RFID_RX_BUFFER[i];
	}

	BP_RFID_RX_BUFFER_COUNT = 0;
	return TRF_NO_ERROR;
}

void BP_ISO15693_Initialize()
{
	BP_RFID_TRF_Set_Protocol(BP_ISO15693_IRQ, TRF_MOD_OOK100, TRF_PROTOCOL_ISO15693 | TRF_PROTOCOL_ISO15693_High_Bit_Rate);  //
	BP_RFID_TRF_Turn_RF_On();
	BP_RFID_TRF_FIFO_Reset();
}
