///////////////////////////////////////////////////////////////////////////////
//
// usb_keyb_structs.c - Data structures defining the USB consumer device.
//
// Code based on TI Stellaris Ware
//
///////////////////////////////////////////////////////////////////////////////

#include "inc/hw_types.h"
#include "usblib/usblib.h"
#include "usblib/usbhid.h"
#include "usblib/usb-ids.h"
#include "usblib/device/usbdevice.h"
#include "usblib/device/usbdhid.h"
#include "usbdhidconsumer.h"
#include "usb_consumer_structs.h"

///////////////////////////////////////////////////////////////////////////////
//
// The languages supported by this device.
//
///////////////////////////////////////////////////////////////////////////////
const unsigned char g_pLangDescriptor[] =
{
    4,
    USB_DTYPE_STRING,
    USBShort(USB_LANG_EN_US)
};

///////////////////////////////////////////////////////////////////////////////
//
// The manufacturer string.
//
///////////////////////////////////////////////////////////////////////////////
const unsigned char g_pManufacturerString[] =
{
    (17 + 1) * 2,
    USB_DTYPE_STRING,
    'T', 0, 'e', 0, 'x', 0, 'a', 0, 's', 0, ' ', 0, 'I', 0, 'n', 0, 's', 0,
    't', 0, 'r', 0, 'u', 0, 'm', 0, 'e', 0, 'n', 0, 't', 0, 's', 0,
};

///////////////////////////////////////////////////////////////////////////////
//
// The product string.
//
///////////////////////////////////////////////////////////////////////////////
const unsigned char g_pProductString[] =
{
    (16 + 1) * 2,
    USB_DTYPE_STRING,
    'C', 0, 'o', 0, 'n', 0, 's', 0, 'u', 0, 'm', 0, 'e', 0, 'r', 0, ' ', 0,
    'c', 0, 'o', 0, 'n', 0, 't', 0, 'r', 0, 'o', 0, 'l', 0
};

///////////////////////////////////////////////////////////////////////////////
//
// The serial number string.
//
///////////////////////////////////////////////////////////////////////////////
const unsigned char g_pSerialNumberString[] =
{
    (8 + 1) * 2,
    USB_DTYPE_STRING,
    '1', 0, '2', 0, '3', 0, '4', 0, '5', 0, '6', 0, '7', 0, '8', 0
};

///////////////////////////////////////////////////////////////////////////////
//
// The interface description string.
//
///////////////////////////////////////////////////////////////////////////////
const unsigned char g_pHIDInterfaceString[] =
{
    (22 + 1) * 2,
    USB_DTYPE_STRING,
    'H', 0, 'I', 0, 'D', 0, ' ', 0, 'C', 0, 'o', 0, 'n', 0, 's', 0,
    'u', 0, 'm', 0, 'e', 0, 'r', 0, ' ', 0, 'I', 0, 'n', 0, 't', 0,
    'e', 0, 'r', 0, 'f', 0, 'a', 0, 'c', 0, 'e', 0
};

///////////////////////////////////////////////////////////////////////////////
//
// The configuration description string.
//
///////////////////////////////////////////////////////////////////////////////
const unsigned char g_pConfigString[] =
{
    (26 + 1) * 2,
    USB_DTYPE_STRING,
    'H', 0, 'I', 0, 'D', 0, ' ', 0, 'c', 0, 'o', 0, 'n', 0, 's', 0,
    'u', 0, 'm', 0, 'e', 0, 'r', 0, ' ', 0, 'C', 0, 'o', 0, 'n', 0,
    'f', 0, 'i', 0, 'g', 0, 'u', 0, 'r', 0, 'a', 0, 't', 0, 'i', 0,
    'o', 0, 'n', 0
};

///////////////////////////////////////////////////////////////////////////////
//
// The descriptor string table.
//
///////////////////////////////////////////////////////////////////////////////
const unsigned char * const g_pStringDescriptors[] =
{
    g_pLangDescriptor,
    g_pManufacturerString,
    g_pProductString,
    g_pSerialNumberString,
    g_pHIDInterfaceString,
    g_pConfigString
};

#define NUM_STRING_DESCRIPTORS (sizeof(g_pStringDescriptors) /                \
                                sizeof(unsigned char *))

///////////////////////////////////////////////////////////////////////////////
//
// The HID consumer control device initialization and customization structures.
//
///////////////////////////////////////////////////////////////////////////////
tHIDconsumerInstance g_HIDConsumerInstance;

const tUSBDHIDconsumerDevice g_sHIDConsumerDevice =
{
    USB_VID_STELLARIS,
    USB_PID_CUSTOM,
    500,
    USB_CONF_ATTR_SELF_PWR | USB_CONF_ATTR_RWAKE,
    HIDConsumerHandler,
    (void *)&g_sHIDConsumerDevice,
    g_pStringDescriptors,
    NUM_STRING_DESCRIPTORS,
    &g_HIDConsumerInstance
};
